/*
IN : packedDataDesc is formatted like this:
	db &c0
	dw &2800, &2900	
	db &c0
	dw &6800, &7b7f
IN : defaultBankAtExit bank to be set (example: &c0)
*/
ExoSetup( packedDataDesc, defaultBankAtExit )
	char *packedDataDesc;
	char defaultBankAtExit;
{
#asm
	di
	pop ix
	pop de
	pop hl
	push hl
	push de
	push ix
	ei

	ld a, e
	call exoSetup
#endasm
}

/*
IN : destPtr is output buffer pointer
IN : length is size to be decompressed to output buffer
OUT : 0=end of stream  1=end not found
*/
ExoStreamUnpack( destPtr, length )
	char *destPtr;
	int length;
{
#asm
	di
	pop ix
	pop bc
	pop de
	push de
	push bc
	push ix
	ei

	call exoUnpack

	ld hl, ( register_de + 1 )
#endasm
}

#asm
exo_mapbasebits: equ &3F00

; A = default bank at exit
; HL = memZones
exoSetup:
	ld ( defaultBankAtExit + 1 ), a

	ld a, ( hl )
	inc hl
	ld ( firstTimeBank + 1 ), a
	ld a, ( hl )
	inc hl
	ld d, ( hl )
	inc hl
	ld e, a
	ld ( memZoneStartPtr + 1 ), de
	ld ( memZonePtr + 1 ), hl

	xor a
	ld ( exoFirstTime + 1 ), a

	ld hl, deexo
	ld ( brkPnt+1 ), hl

	ret

; DE = destination
; BC = size of bytes to unpack
exoUnpack:		
	ld ( lngAdr ), bc
		
exoFirstTime:
	ld a, 0
	or a
	jp z, skipExoFirstTime

register_ix:	ld ix, 0
register_de:	ld de, 0
register_bc:	ld bc, 0
register_hl:	ld hl, 0

	ld iy, exo_mapbasebits
	jp endExoFirstTime
	
skipExoFirstTime:
	
memZoneStartPtr:
	ld hl, 0
	
endExoFirstTime:

firstTimeBank:
	ld a, 0
	call setBank

	ld a, 1
	ld ( exoFirstTime + 1 ), a
	
brkPnt:
	call deexo
	
	ld ( register_ix + 2 ), ix
	ld ( register_hl + 1 ), hl
	ld ( register_de + 1 ), de
	ld ( register_bc + 1 ), bc	
	ld ( brkPnt + 1 ), iy
	
defaultBankAtExit:
	ld a, 0
	call setBank

	ld a, 1
	or a
	ret	

endDep:
	pop hl

	ld a, ( defaultBankAtExit + 1 )
	call setBank

	xor a
	or a
	ret
	
deexo: 
	ld ixh, 128
	ld b, 52
  
	ld iy, exo_mapbasebits
	push de
  
exo_initbits: 
	ld a, b
	sub 4
	and 15
	jr nz,exo_node1
	ld de, 1
exo_node1:
	ld c,16
exo_get4bits: 
	call exo_getbit
	rl c
	jr nc, exo_get4bits
	ld ( iy ), c

	push hl
	inc c
	ld hl, 0
	scf
exo_setbit: 
	adc hl,hl
	dec c
	jr nz,exo_setbit
	ld ( iy + 52 ), e
	ld ( iy + 104 ), d
	add hl,de
	ex de,hl
	inc iy

	pop hl
	dec b
	jp nz, exo_initbits
	pop de

exo_literalcopy:
	ldi
	ld bc,( lngAdr )
	dec bc
	ld ( lngAdr ),bc
	ld a, b
	or c
	jr nz, exo_mainloop
	ld iy, exo_mainloop
	ret

exo_mainloop:
	call exo_getbit
	jr c, exo_literalcopy
	call Raster
	ld c, 255
exo_getindex: 
	inc c
	call exo_getbit
	jr nc, exo_getindex
	ld a, c
	cp 16
	jr z, endDep
	push de
	call exo_getpair
	ld ( exo_length + 1 ), bc
	ld de, 512+48
	dec bc
	ld a, b
	or c
	jr z, exo_goforit
	ld de, 1024+32
	dec bc
	ld a,b
	or c
	jr z, exo_goforit
	ld e, 16
exo_goforit: 
	call exo_getbits
exoDoRaster:
	ld a, 1
	dec a
	jp nz, exoDoRasterSkip
	call Raster
	ld a, 6	
exoDoRasterSkip:
	ld (exoDoRaster+1), a
	ld a, e
	add a, c
	ld c, a
	call exo_getpair
	pop de
	ld ( savUncr ), hl
	ld h, d
	ld l, e
	sbc hl,bc
exo_length:
	ld bc, 0
Label1:
	push hl
	ld hl,( lngAdr )
	or a
	sbc hl, bc
	jr z, Label2
	jr c, Label3
	ld ( lngAdr ), hl
	pop hl
	ldir
	call Raster
	ld hl,( savUncr )
	jr exo_mainloop
Label2:
	pop hl
	ldir
	ld hl,( savUncr )
	ld iy, exo_mainloop
    ret
Label3:
	add hl, bc
	ld a, c
	sub l
	ld c, a
	ld a, b
	db &9C ;sbc h
	ld b, a
	ld ( lngAdr ),bc
	ld b, h
	ld c, l
	pop hl
	ldir
	ld bc,( lngAdr )
	ld iy, Label1
	ret
exo_getpair:
	ld iyl, c
	ld d,( iy )
	call exo_getbits
	ld a, c
	add a,( iy + 52 )
	ld c, a
	ld a, b
	adc a,( iy + 104 )
	ld b, a
	ret
exo_getbits: 
	ld bc, 0
exo_gettingbits:
	dec d
	ret m
	call exo_getbit
	rl c
	rl b
	jr exo_gettingbits
exo_getbit: 
	ld a, ixh
	add a, a
	ld ixh, a
	ret nz
	
	push af

	ld ( memZonePtrDE + 1 ), de
	ld ( memZonePtrHL + 1 ), hl

memZonePtr:
	ld de, 0
	
	call Raster

	ld a, ( de )
	cp l
	jr nz, skipNextZonePtr	
	inc de
	ld a, ( de )	
	cp h
	jr nz, skipNextZonePtr	
	inc de

	;db &ed, &ff

	ex de, hl

	ld a, ( hl )
	ld ( firstTimeBank + 1 ), a
	inc hl
	call setBank
	
	ld a, ( hl )
	ld e, a
	inc hl
	ld a, ( hl )
	ld d, a
	inc hl
	
	ld ( memZonePtr + 1 ), hl
	ld ( memZonePtrHL + 1 ), de
		
skipNextZonePtr:

memZonePtrDE:
	ld de, 0

memZonePtrHL:
	ld hl, 0

	pop af

deb1:
	ld a,( hl )
	inc hl
	rla
	ld ixh, a
	ret
	
setBank:
	push bc
	ld bc, &7f00
	out (c), c
	out (c), a
	pop bc

	ret
	
lngAdr:
	dw 0
savUncr:
	dw 0	
#endasm
