
SetColor( pen, color )
	char pen;
	char color;
{
#asm
	di
	pop bc
	pop hl
	pop de
	
	push de
	push hl
	push bc
	ei

; E = pen
; L = color
asmSetColor:

	di
	ld b, &7f
	ld c, e
	out (c), c
	ld c, l
	out (c), c
	ei
#endasm
}

SetPalette( palette )
	char *palette;
{
#asm
	di
	pop bc
	pop hl
	push hl
	push bc
	ei

; HL = palette in memory
asmSetPalette:

	ld e, 0

setPaletteBegin:
	push de
	push hl

	ld a, (hl)
	ld l, a
	;db &ed, &ff
	call asmSetColor

	pop hl
	pop de
	
	inc hl

	inc e
	ld a, e
	cp 16
	jp nz, setPaletteBegin
#endasm	
}

SetMode( mode )
	char mode;
{
#asm
	di
	pop bc
	pop de
	push de
	push bc
	ei

; E = mode
asmSetMode:

	ld a, e
	
	di
	ld bc, &7f00
	out (c), a
	ei
#endasm
}

#define STACKBANKSIZE 16
static int stackBanks[ STACKBANKSIZE ];
static char stackBankIndex;

#asm
curBank:
	db &0
#endasm

UpdateBank( bank )
	char bank;
{
#asm
	;db &ed, &ff
#endasm

#asm
	; interrupts compliant

	ld ( updateBankSP + 1 ), sp
updateBankSP:
	ld hl, 0
	inc hl
	inc hl
	ld a, (hl)

	ld ( curBank ), a
	
	di
	ld bc, &7f00
	out (c), c
	out (c), a
	ei
#endasm

	stackBanks[ stackBankIndex - 1 ] = bank;
}

// never calls directly SetBank, use PushBank/PopBank instead !
SetBank( bank )
	char bank;
{
#asm
	; interrupts compliant
	
	ld ( setBankSP + 1 ), sp
setBankSP:
	ld hl, 0
	inc hl
	inc hl
	ld a, (hl)
	
; E = bank
asmSetBank:

	ld bc, &7f00
	out (c), c
	out (c), a

	ld ( curBank ), a
#endasm
}

PushBank( bank )
	char bank;
{
	stackBanks[ stackBankIndex ] = bank;
	
	stackBankIndex++;
	if ( stackBankIndex == STACKBANKSIZE )
	{
#asm
	db &ed, &ff
#endasm
	}
	
	SetBank( bank );
}

// output (L register for asm) = bank
PopBank()
{
	char bank;

	stackBankIndex--;
	if ( stackBankIndex == 0 )
	{
		bank = BANKC0;
	}
	else
	{
		bank = stackBanks[ stackBankIndex - 1 ];
	}
	
	SetBank( bank );

	return bank;
}

// output (L register for asm) = bank
GetBank()
{
	return stackBanks[ stackBankIndex - 1 ];
}
