
static int textCount;

wmemcpy( destination, source, length )
	char *destination;
	char *source;
	int length;
{
#asm
	di
	pop ix
	pop bc
	pop hl
	pop de
	push de
	push hl
	push bc
	push ix
	ei
	
doMemCpy:
	ld a, (hl)
	inc hl
	ld (de), a
	inc de
	dec bc
	ld a, b
	or a
	call z, Raster
	or c
	jp nz, doMemCpy
#endasm
}

WriterUnpack( source, destination )
	char *source;
	char *destination;
{
#asm
	di
	pop bc
	pop de
	pop hl
	push hl
	push de
	push bc
	ei

; HL = source
; DE = destination
asmUnpack:
	ld a, ( hl )
	inc hl

	; unpacked size
	inc hl
	inc hl

	cp 1
	jp z, BitBusterUnpack
#endasm
}

BitBusterUnpack()
{
#asm
	inc hl
	inc hl
	inc hl
	inc hl

	ld a, 128
	
	exx
	ld de, 1
	exx

depack_loop:
	add a, a
	jr nz, getbit1

	ld a, ( hl )
	inc hl
	rla

getbit1:
	jr c, output_compressed
	ldi

	add a, a
	jr nz, getbit0

	ld a, ( hl )
	inc hl
	rla

getbit0:
	jr c, output_compressed
	ldi
	
	add a, a
	jr nz, getbit2

	ld a, ( hl )
	inc hl
	rla

getbit2:
	jr c, output_compressed
	ldi

	add a, a
	jr nz, getbit3

	ld a, ( hl )
	inc hl
	rla

getbit3:
	jr c, output_compressed
	ldi
	jr depack_loop

output_compressed:
	ld c, ( hl )
	inc hl

output_match:
	ld b, 0
	bit 7, c
	jr z, output_match1

	add a, a
	jr nz, getbit4

	ld a, ( hl )
	inc hl
	rla

getbit4:
	rl b

	add a, a
	jr nz, getbit5

	ld a, ( hl )
	inc hl
	rla

getbit5:
	rl b

	add a, a
	jr nz, getbit6

	ld a, ( hl )
	inc hl
	rla

getbit6:
	rl b

	add a, a
	jr nz, getbit7

	ld a, ( hl )
	inc hl
	rla

getbit7:
	jr c, output_match1
	res 7, c

output_match1:
	inc bc

	exx
	ld h, d
	ld l, e
	ld b, e

get_gamma_value_size:
	exx

	add a, a
	jr nz, getbit8

	ld a, ( hl )
	inc hl
	rla

getbit8:
	exx
	jr nc, get_gamma_value_size_end
	inc b
	jr get_gamma_value_size

get_gamma_value_bits:
	exx

	add a, a
	jr nz, getbit9

	ld a, ( hl )
	inc hl
	rla

getbit9:
	exx
	adc hl, hl

get_gamma_value_size_end:
	djnz get_gamma_value_bits

get_gamma_value_end:
	inc hl
	exx

	ret c

	push hl

	exx
	push hl
	exx

	ld h, d
	ld l, e
	sbc hl, bc

	pop bc

	ldir

	call Raster

	pop hl

	add a, a
	jr nz, getbita

	ld a, ( hl )
	inc hl
	rla

getbita:
	jr c, output_compressed
	ldi

	add a, a
	jr nz, getbitb

	ld a, ( hl )
	inc hl
	rla

getbitb:
	jr c, output_compressed
	ldi
	jp depack_loop	
#endasm
}

#asm
Text1:
#endasm
#include "SOTA.cpctext.cxx"

#asm
Text2:
#endasm
#include "SOTA2.cpctext.cxx"

#asm
Text3:
#endasm
#include "SOTA3.cpctext.cxx"

#asm
Text4:
#endasm
#include "SOTA4.cpctext.cxx"

GetTextPtr()
{
#asm
	ld hl, Text1
#endasm
}

GetText2Ptr()
{
#asm
	ld hl, Text2
#endasm
}

GetText3Ptr()
{
#asm
	ld hl, Text3
#endasm
}

GetText4Ptr()
{
#asm
	ld hl, Text4
#endasm
}

InitWriter()
{
	textCount = 3;
}

ResetWriter()
{
	showWriterCount = 5;
	writerFrameCount = 0;

	textCount++;
	if ( textCount == 4 )
	{
		textCount = 0;
	}
}

ShowText( x, y, textPtr )
	int x;
	int y;
	char *textPtr;
{
	char c;
	char *fontPtr;

	int prevX;

	prevX = x;
	
	for ( ; ; )
	{
		c = *textPtr;
		textPtr++;
		
		if	( c == -1 )
		{
			return;
		}

		if ( c == -2 ) // NEW LINE
		{
			x = prevX;
			y += 21;
		}
		else
		{	
			fontPtr = WRITERFONTBMPSPRDATA1_PTRINBANK + 2; // skip xy
			fontPtr += c;
			fontPtr += c;
			fontPtr += c;
			fontPtr += c;
			fontPtr += c;
			fontPtr += c;

			PushBank( WRITERFONTBMPSPRDATA1_BANK );
			DrawSpriteDataScanline( x, y, fontPtr, fontPtr, WRITERFONTBMPSPRDATA1_BANK, 6, 21, 228 );
			PopBank();
			
			x += 6;
		}
	}
}

DrawWriter()
{
	char *ptr;

	if ( textCount == 0 )
	{
		ptr = GetTextPtr();
	}
	else if ( textCount == 1 )
	{
		ptr = GetText2Ptr();
	}
	else if ( textCount == 2 )
	{
		ptr = GetText3Ptr();
	}
	else
	{
		ptr = GetText4Ptr();
	}

	ShowText( 6, 28, ptr );
}

DisplayTextFrame()
{
	if ( writerFrameCount != 0 )
	{
		BringBackToFrontLines();
		InitSpriteFrame();
		DrawWriter();		
		BringBackToFrontLines();
		InitSpriteFrame();
		DrawWriter();
	}
}

ShowWriter()
{
	if ( writerFrameCount == 1 )
	{
		PushBank( WRITERCPCBITMAPPALETTE_BANK );
		SetPalette( WRITERCPCBITMAPPALETTE_PTRINBANK );
		PopBank();	
	}

	DisplayTextFrame();

	if ( writerFrameCount < 2 )
	{
		PushBank( WRITERBACKGROUNDBMPBOTTOMBINPCK_BANK );
		wmemcpy( GetTopBackVideoPtr(), WRITERBACKGROUNDBMPBOTTOMBINPCK_PTRINBANK, WRITERBACKGROUNDBMPBOTTOMBINPCK_SIZE );
		PopBank();
		WriterUnpack( GetTopBackVideoPtr(), GetBottomBackVideoPtr() );

		PushBank( WRITERBACKGROUNDBMPTOPBINPCK_BANK );
		WriterUnpack( WRITERBACKGROUNDBMPTOPBINPCK_PTRINBANK, GetTopBackVideoPtr() );
		PopBank();
	}

	DisplayTextFrame();

	writerFrameCount++;

#asm
	; play music
	ld a, 1
	ld ( &0040 ), a
#endasm
}